@@ -1,28 +1,41 @@
 ;;;
 ;;; gl/math3d.scm - auxiliary vector arithmetics for 3D graphics
 ;;;
-;;;  Copyright (c) 2002-2008  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2002-2014  Shiro Kawai  <shiro@acm.org>
 ;;;
-;;;  Permission to use, copy, modify, distribute this software and
-;;;  accompanying documentation for any purpose is hereby granted,
-;;;  provided that existing copyright notices are retained in all
-;;;  copies and that this notice is included verbatim in all
-;;;  distributions.
-;;;  This software is provided as is, without express or implied
-;;;  warranty.  In no circumstances the author(s) shall be liable
-;;;  for any damages arising out of the use of this software.
+;;;  Redistribution and use in source and binary forms, with or without
+;;;  modification, are permitted provided that the following conditions
+;;;  are met:
 ;;;
-;;;  $Id: math3d.scm,v 1.14 2008-06-06 00:58:26 shirok Exp $
+;;;   1. Redistributions of source code must retain the above copyright
+;;;      notice, this list of conditions and the following disclaimer.
+;;;
+;;;   2. Redistributions in binary form must reproduce the above copyright
+;;;      notice, this list of conditions and the following disclaimer in the
+;;;      documentation and/or other materials provided with the distribution.
+;;;
+;;;   3. Neither the name of the authors nor the names of its contributors
+;;;      may be used to endorse or promote products derived from this
+;;;      software without specific prior written permission.
+;;;
+;;;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+;;;  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+;;;  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+;;;  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+;;;  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+;;;  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+;;;  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+;;;  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+;;;  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+;;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+;;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
 (define-module gl.math3d
   (use gauche.uvector)
   (use gauche.sequence)
-  (export-all)
-  )
-
+  (export-all))
 (select-module gl.math3d)
-
 (dynamic-load "libgauche-math3d" :export-symbols #t)
 
 (define-reader-ctor 'vector4f vector4f)
@@ -31,24 +44,22 @@
 (define-reader-ctor 'quatf    quatf)
 
 (define-reader-ctor 'vector4f-array
-  (lambda (length . vectors)
-    (let1 arr (make-vector4f-array length)
-      (let loop ((i 0)
-                 (v vectors))
+  (^[length . vectors]
+    (rlet1 arr (make-vector4f-array length)
+      (let loop ([i 0]
+                 [v vectors])
         (when (< i length)
           (vector4f-array-set! arr i (list->vector4f (car v)))
-          (loop (+ i 1) (cdr v))))
-      arr)))
+          (loop (+ i 1) (cdr v)))))))
 
 (define-reader-ctor 'point4f-array
-  (lambda (length . points)
-    (let1 arr (make-point4f-array length)
+  (^[length . points]
+    (rlet1 arr (make-point4f-array length)
       (let loop ((i 0)
                  (p points))
         (when (< i length)
           (point4f-array-set! arr i (list->point4f (car p)))
-          (loop (+ i 1) (cdr p))))
-      arr)))
+          (loop (+ i 1) (cdr p)))))))
 
 ;;=================================================================
 ;; Auxiliary fns
@@ -63,27 +74,25 @@
   (f32vector->point4f-array/shared (f32vector-copy v)))
 
 (define (list->vector4f-array l)
-  (let* ((len (length l))
-         (a   (make-vector4f-array len))
-         (i   0))
-    (for-each (lambda (elt)
-                (unless (vector4f? elt)
-                  (error "vector4f required, but got" elt))
-                (vector4f-array-set! a i elt)
-                (inc! i))
-              l)
+  (let* ([len (length l)]
+         [a   (make-vector4f-array len)]
+         [i   0])
+    (dolist [elt l]
+      (unless (vector4f? elt)
+        (error "vector4f required, but got" elt))
+      (vector4f-array-set! a i elt)
+      (inc! i))
     a))
 
 (define (list->point4f-array l)
-  (let* ((len (length l))
-         (a   (make-point4f-array len))
-         (i   0))
-    (for-each (lambda (elt)
-                (unless (point4f? elt)
-                  (error "point4f required, but got" elt))
-                (point4f-array-set! a i elt)
-                (inc! i))
-              l)
+  (let* ([len (length l)]
+         [a   (make-point4f-array len)]
+         [i   0])
+    (dolist [elt l]
+      (unless (point4f? elt)
+        (error "point4f required, but got" elt))
+      (point4f-array-set! a i elt)
+      (inc! i))
     a))
 
 ;;=================================================================
@@ -197,15 +206,15 @@
 ;; generate more efficient code.
 (define-syntax %math3dobj-iterator
   (syntax-rules ()
-    ((_ (size-expr . args) ref obj proc)
+    [(_ (size-expr . args) ref obj proc)
      (let ((len (size-expr . args))
            (i 0))
-       (proc (lambda () (>= i len))
-             (lambda () (begin0 (ref obj i) (inc! i))))))
-    ((_ len ref obj proc)
+       (proc (^[] (>= i len))
+             (^[] (begin0 (ref obj i) (inc! i)))))]
+    [(_ len ref obj proc)
      (let ((i 0))
-       (proc (lambda () (>= i len))
-             (lambda () (begin0 (ref obj i) (inc! i))))))
+       (proc (^[] (>= i len))
+             (^[] (begin0 (ref obj i) (inc! i)))))]
     ))
 
 (define-method call-with-iterator ((v <vector4f>) proc . args)
@@ -230,9 +239,9 @@
 (define-syntax %math3dobj-builder
   (syntax-rules ()
     ((_ len new set proc)
-     (let ((v (new)) (i 0))
-       (proc (lambda (item) (when (>= i len) (set v i item) (inc! i)))
-             (lambda () v))))))
+     (let ([v (new)] [i 0])
+       (proc (^[item] (when (>= i len) (set v i item) (inc! i)))
+             (^[] v))))))
      
 (define-method call-with-builder ((class <vector4f-meta>) proc . args)
   (%math3dobj-builder 4 make-vector4f vector4f-set! proc))
@@ -248,16 +257,14 @@
 
 (define-syntax %math3dobj-builder*
   (syntax-rules ()
-    ((_ new list-> set proc args)
-     (let ((size (get-keyword :size args #f)))
-       (if size
-           (let ((v (new size)) (i 0))
-             (proc (lambda (item)
-                     (when (< i size) (set v i item) (inc! i)))
-                   (lambda () v)))
-           (let ((r '()))
-             (proc (lambda (item) (push! r item))
-                   (lambda () (list-> r)))))))))
+    [(_ new list-> set proc args)
+     (if-let1 size (get-keyword :size args #f)
+       (let ([v (new size)] [i 0])
+         (proc (^[item] (when (< i size) (set v i item) (inc! i)))
+               (^[] v)))
+       (let ([r '()])
+         (proc (^[item] (push! r item))
+               (^[] (list-> r)))))]))
 
 (define-method call-with-builder ((class <vector4f-array-meta>) proc . args)
   (%math3dobj-builder* make-vector4f-array list->vector4f-array
@@ -310,6 +317,3 @@
   (matrix4f-mul m s))
 (define-method object-* ((x <quatf>) (y <quatf>))
   (quatf-mul x y))
-
-
-(provide "gl/math3d")
